﻿namespace TabsStudioShaper
{
    public class TabShape : System.Windows.Controls.Decorator
    {
        protected override System.Windows.Size MeasureOverride(System.Windows.Size sizeAvailable)
        {
            double width = 0;
            double boundsWidth = GetRightSideBounds().Right - GetLeftSideBounds().Left;
            if (Child != null)
            {
                double availableWidth = System.Math.Max(sizeAvailable.Width - boundsWidth, 0);
                Child.Measure(new System.Windows.Size(availableWidth, sizeAvailable.Height));
                width = Child.DesiredSize.Width;
            }
            return new System.Windows.Size(width + boundsWidth, GetHeight());
        }

        protected override System.Windows.Size ArrangeOverride(System.Windows.Size sizeArrange)
        {
            if (Child != null)
            {
                double left = -GetLeftSideBounds().Left;
                double top = (sizeArrange.Height - Child.DesiredSize.Height) / 2;
                double width =
                    System.Math.Max(sizeArrange.Width + GetLeftSideBounds().Left - GetRightSideBounds().Right, 0);
                double height = Child.DesiredSize.Height;
                Child.Arrange(new System.Windows.Rect(left, top, width, height));
            }
            return sizeArrange;
        }

        protected override void OnRender(System.Windows.Media.DrawingContext dc)
        {
            double halfPenWidth = 0.0;
            if (Border != null)
                halfPenWidth = Border.Thickness / 2.0;
            System.Windows.Media.GuidelineSet guidelines = new System.Windows.Media.GuidelineSet();
            guidelines.GuidelinesX.Add(0 + halfPenWidth);
            guidelines.GuidelinesX.Add(GetLeftSideBounds().Width);
            guidelines.GuidelinesX.Add(RenderSize.Width - GetRightSideBounds().Width);
            guidelines.GuidelinesX.Add(RenderSize.Width + halfPenWidth);
            guidelines.GuidelinesY.Add(0 + halfPenWidth);
            guidelines.GuidelinesY.Add(GetHeight());
            dc.PushGuidelineSet(guidelines);
            {
                double width = RenderSize.Width - GetLeftSideBounds().Width - GetRightSideBounds().Width;
                System.Windows.Media.PathGeometry g = new System.Windows.Media.PathGeometry();
                System.Windows.Media.PathFigure f = new System.Windows.Media.PathFigure();
                f.StartPoint = new System.Windows.Point(0, 0);
                f.Segments.Add(new System.Windows.Media.LineSegment(
                        new System.Windows.Point(width, 0), true));
                f.Segments.Add(new System.Windows.Media.LineSegment(
                        new System.Windows.Point(width, GetHeight()),false));
                f.Segments.Add(new System.Windows.Media.LineSegment(
                        new System.Windows.Point(0, GetHeight()), false));
                g.Figures.Add(f);

                System.Windows.Media.TranslateTransform transform =
                    new System.Windows.Media.TranslateTransform(GetLeftSideBounds().Width, 0);
                dc.PushTransform(transform);
                dc.DrawGeometry(Background, Border, g);
                dc.Pop();
            }
            if (LeftSide != null)
            {
                System.Windows.Media.TranslateTransform transform =
                    new System.Windows.Media.TranslateTransform(-GetLeftSideBounds().Left, 0);
                dc.PushTransform(transform);
                dc.DrawGeometry(Background, Border, LeftSide);
                dc.Pop();
            }
            if (RightSide != null)
            {
                System.Windows.Media.TranslateTransform transform =
                    new System.Windows.Media.TranslateTransform(
                        RenderSize.Width - GetRightSideBounds().Width - GetRightSideBounds().Left, 0);
                dc.PushTransform(transform);
                dc.DrawGeometry(Background, Border, RightSide);
                dc.Pop();
            }
            dc.Pop();
        }

        private System.Windows.Rect GetLeftSideBounds()
        {
            if (LeftSide == null)
                return new System.Windows.Rect(0, 0, 0, defaultHeight);

            return LeftSide.Bounds;
        }

        private System.Windows.Rect GetRightSideBounds()
        {
            if (RightSide == null)
                return new System.Windows.Rect(0, 0, 0, defaultHeight);

            return RightSide.Bounds;
        }

        private double GetHeight()
        {
            return GetLeftSideBounds().Height;
        }

        public System.Windows.Media.Geometry LeftSide
        {
            get { return (System.Windows.Media.Geometry)GetValue(LeftSideProperty); }
            set { SetValue(LeftSideProperty, value); }
        }

        public System.Windows.Media.Geometry RightSide
        {
            get { return (System.Windows.Media.Geometry)GetValue(RightSideProperty); }
            set { SetValue(RightSideProperty, value); }
        }

        public System.Windows.Media.Brush Background
        {
            get { return (System.Windows.Media.Brush)GetValue(BackgroundProperty); }
            set { SetValue(BackgroundProperty, value); }
        }

        public System.Windows.Media.Pen Border
        {
            get { return (System.Windows.Media.Pen)GetValue(BorderProperty); }
            set { SetValue(BorderProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty LeftSideProperty =
            System.Windows.DependencyProperty.Register("LeftSide", typeof(System.Windows.Media.Geometry),
                typeof(TabShape), new System.Windows.FrameworkPropertyMetadata(null,
                    System.Windows.FrameworkPropertyMetadataOptions.AffectsMeasure));

        public static readonly System.Windows.DependencyProperty RightSideProperty =
            System.Windows.DependencyProperty.Register("RightSide", typeof(System.Windows.Media.Geometry),
                typeof(TabShape), new System.Windows.FrameworkPropertyMetadata(null,
                    System.Windows.FrameworkPropertyMetadataOptions.AffectsMeasure));

        public static readonly System.Windows.DependencyProperty BackgroundProperty =
            System.Windows.DependencyProperty.Register("Background", typeof(System.Windows.Media.Brush),
                typeof(TabShape), new System.Windows.FrameworkPropertyMetadata(null, 
                    System.Windows.FrameworkPropertyMetadataOptions.AffectsRender));

        public static readonly System.Windows.DependencyProperty BorderProperty =
            System.Windows.DependencyProperty.Register("Border", typeof(System.Windows.Media.Pen),
                typeof(TabShape), new System.Windows.FrameworkPropertyMetadata(
                    new System.Windows.Media.Pen(System.Windows.Media.Brushes.DarkGray, 1),
                    System.Windows.FrameworkPropertyMetadataOptions.AffectsRender));

        private static readonly double defaultHeight = 24.0;
    }
}
