﻿namespace TabsStudioShaper
{
    class ZOrderManager
    {
        public ZOrderManager(TabsStudioExt.ITabsStudioEngine engine_)
        {
            timer = new System.Windows.Threading.DispatcherTimer();
            timer.Interval = System.TimeSpan.FromMilliseconds(100);
            timer.Tick += new System.EventHandler(TimerTick);

            engine = engine_;
            panels = new System.Collections.Generic.List<System.Windows.Controls.Panel>();
            engine.TabsCreated += OnTabsCreated;
            engine.TabsDestroyed += OnTabsDestroyed;
        }

        public void Close()
        {
            engine.TabsCreated -= OnTabsCreated;
            engine.TabsDestroyed -= OnTabsDestroyed;
        }

        private void OnTabsCreated(object sender, TabsStudioExt.TabsEventArgs e)
        {
            e.Tabs.TabPanel2.LayoutUpdated += OnLayoutUpdated;
            panels.Add(e.Tabs.TabPanel2);
        }

        private void OnTabsDestroyed(object sender, TabsStudioExt.TabsEventArgs e)
        {
            e.Tabs.TabPanel2.LayoutUpdated -= OnLayoutUpdated;
            panels.Remove(e.Tabs.TabPanel2);
        }

        private void TimerTick(object sender, System.EventArgs e)
        {
            timer.Stop();
            UpdateOrder();
        }

        private void OnLayoutUpdated(object sender, System.EventArgs e)
        {
            timer.Start();
        }

        private void UpdateOrder()
        {
            foreach (System.Windows.Controls.Panel panel in panels)
            {
                int zindex = panel.Children.Count;
                foreach (System.Windows.UIElement i in panel.Children)
                {
                    if (isTabSelectedDescriptor == null)
                        isTabSelectedDescriptor = System.ComponentModel.DependencyPropertyDescriptor.FromName(
                            "IsTabSelected", i.GetType(), i.GetType());

                    System.Windows.Controls.Panel.SetZIndex(i, 
                        (bool)isTabSelectedDescriptor.GetValue(i) ? panel.Children.Count + 1 : zindex);
                    --zindex;
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private System.Collections.Generic.List<System.Windows.Controls.Panel> panels;
        private System.ComponentModel.DependencyPropertyDescriptor isTabSelectedDescriptor;
        private System.Windows.Threading.DispatcherTimer timer;
    }
}
